#if !defined(LKT_NET_H__)
#define LKT_NET_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <lektor/database.h>
#include <lektor/common.h>
#include <lektor/queue.h>
#include <lektor/mkv.h>
#include <lektor/config.h>
#include <lektor/reg.h>
#include <lektor/uri.h>

#include <mpv/client.h>
#include <curl/curl.h>

struct lkt_state;
struct lkt_command {
    bool was_truncated;
    char *name;
    char *args[LKT_MESSAGE_ARGS_MAX];
    long cont;
};

/* Create and destruct a command structure. */
struct lkt_command lkt_command_parse(char *raw);
struct lkt_command lkt_command_clone(struct lkt_command *c);
void lkt_command_free(struct lkt_command *c);

/* Print debug and warning informations about the passed command. */
void lkt_command_log_to_debug(const struct lkt_command *);

/* Messages that are send and recieved */
struct lkt_message;

/* Create a message structure. */
struct lkt_message *lkt_message_new(void);
struct lkt_message *lkt_message_new_str(const char *msg);
struct lkt_message *lkt_message_new_fmt(const char *fmt, ...);
struct lkt_message *lkt_message_clone(struct lkt_message *msg);

/* Destruct a message structure. */
void lkt_message_free(struct lkt_message *const msg);

/* Send a Message into a file descriptor. You need to check the returned value
 * and errno manually if n <= 0 as you would when calling the 'send' function.
 * This is only a wrapper for 'send(fd, data, len, MSG_NOSIGNAL)'. */
int lkt_message_send(int fd, const struct lkt_message *const msg);

struct lkt_state {
    struct pollfd *fds;
    struct lkt_client *clients;
    size_t fds_len;
    size_t fds_max;
    char host[HOST_NAME_MAX];
    char port[LKT_LINE_MAX];
    struct queue queue;

    /* The database */
    lkt_db *db;
    const char *kara_prefix;
    uint64_t update_total;
    uint64_t update_current;
    uint8_t is_updating;

    /* Internal event flags */
    MPD_IDLE_FLAG mpd_idle_events;
    pthread_mutex_t lock;

    /* Modules */
    struct lkt_module window_mod;
    struct lkt_module repo_mod;

    /* For uptime */
    time_t start_date;

    /* Signal handlers for this server.
     * Catch them all! */
    void (*___signal_INT)(struct lkt_state *);
    void (*___signal_ILL)(struct lkt_state *);
    void (*___signal_QUIT)(struct lkt_state *);
    void (*___signal_USR1)(struct lkt_state *);
    void (*___signal_USR2)(struct lkt_state *);
};

/* Send a message to the connected client. */
void lkt_state_send(struct lkt_state *srv, size_t c, struct lkt_message *msg);

/* Get and set continuation state for a client. */
size_t lkt_remaining_msg(struct lkt_state *srv, size_t c);

int lkt_get_continuation(struct lkt_state *srv, size_t c);
void lkt_set_continuation(struct lkt_state *srv, size_t c, int i);

/* Get the mask to watch for events for a given client, to be used with the `idle` command.
 * Return a pointer to the client's mask */
int lkt_client_get_mask(struct lkt_state *srv, size_t c);
void lkt_client_add_mask(struct lkt_state *srv, size_t c, MPD_IDLE_FLAG add);
void lkt_client_clear_mask(struct lkt_state *srv, size_t c);

/* Register a client as authentificated. */
bool lkt_client_auth(struct lkt_state *srv, size_t c, bool set_auth);

/* The server's listen function. */
EXIT_FUNCTION lkt_listen(struct lkt_state *srv);

/* Close all fds for the server */
void lkt_close_server(struct lkt_state *srv);

#if defined(__cplusplus)
}
#endif
#endif /* LKT_NET_H__ */
