mod command_deamon;
mod status_deamon;
pub use command_deamon::*;
pub use status_deamon::*;

use lkt_lib::*;
use log::*;
use std::{
    cell::Cell,
    io,
    sync::{
        atomic::{self, AtomicBool},
        mpsc::{channel, Receiver, RecvTimeoutError, Sender},
        Arc, Mutex,
    },
    thread,
    time::Duration,
};

/// A common interface for all deamons.
pub trait Deamon: Sized {
    type Channels;

    /// Quit the deamon
    fn quit(&self);

    /// Tel whever the quit has been issued.
    fn should_quit(&self) -> bool;

    /// Spawn a deamon
    fn spawn(hostname: String, port: i16) -> io::Result<(Self::Channels, Self)>;

    /// Returns true when the thread has terminated.
    fn joined(&self) -> bool;

    /// Join the deamon.
    fn join(&self);
}

#[macro_export]
macro_rules! break_when_flagged {
    ($arc_atomic: expr) => {
        if $arc_atomic.load(atomic::Ordering::SeqCst) {
            break;
        }
    };
}

#[macro_export]
macro_rules! implement_deamon_quit {
    () => {
        fn quit(&self) {
            debug!("asked to quit a deamon!");
            self.quit.store(true, atomic::Ordering::SeqCst);
        }

        fn should_quit(&self) -> bool {
            self.quit.load(atomic::Ordering::SeqCst)
        }
    };
}

#[macro_export]
macro_rules! implement_deamon_joined {
    () => {
        fn joined(&self) -> bool {
            return self.joined.load(atomic::Ordering::SeqCst);
        }

        fn join(&self) {
            let locked = self.thread.lock();
            if locked.is_err() {
                error!("Failed to lock the mutex that has the join handler",)
            }
            let locked = locked.unwrap();
            let thread = locked.replace(None);
            match thread {
                Some(thread) => {
                    let _ = thread.join();
                    self.joined.store(true, atomic::Ordering::SeqCst);
                }
                None => error!("Nothing to join!"),
            }
        }
    };
}

pub(self) use break_when_flagged;
pub(self) use implement_deamon_joined;
pub(self) use implement_deamon_quit;
